@@ -15,10 +15,31 @@
 //     along with offlinefs.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "source.hxx"
+#include "single.hxx"
+#include "types.hxx"
 
 using std::string;
+using std::auto_ptr;
+namespace off = offlinefs;
 
-Source::Source(FsTxn& txns,uint64_t fileid,int mode):dbs(txns.dbs),fileid(fileid),size(0),mode(mode) {
-   File f(txns,fileid);
-   size=f.getattr<off_t>("offlinefs.size");
+Source::Source(File& f,int mode):dbs(f.txns.dbs),fileid(f.getid()),size(0),mode(mode) {
+   size=f.getattr<off::off_t>("offlinefs.size");
+}
+
+auto_ptr<Source> Source::getsource(File& f,int mode){
+   string source = f.getattrv("offlinefs.source");
+
+   if(source == "single")
+      return auto_ptr<Source>(new Source_single(f,mode));
+   else
+      throw std::runtime_error(string("Source::getsource: Unknown source type \"")+ source + "\".");
+}
+
+void Source::remove(File& f){
+   string source = f.getattrv("offlinefs.source");
+
+   if(source == "single")
+      Source_single::remove(f);
+   else
+      throw std::runtime_error(string("Source::remove: Unknown source type \"")+ source + "\".");
 }
